@@ -589,8 +589,6 @@ again:
 			/* fall through */
 		case 5: /* read key/value pair */
 			ret = 0;
-			_SCRUB_KVREAD(ret, &i, data_extents_scrubbed, avail, l,
-					&p[curr]->p);
 			_SCRUB_KVREAD(ret, &i, data_extents_scrubbed, avail, l,
 					&p[curr]->p);
 			_SCRUB_KVREAD(ret, &i, tree_extents_scrubbed, avail, l,
@@ -849,8 +847,7 @@ static void *scrub_one_dev(void *ctx)
 		      IOPRIO_PRIO_VALUE(sp->ioprio_class,
 					sp->ioprio_classdata));
 	if (ret)
-		warning("setting ioprio failed: %s (ignored)",
-			strerror(errno));
+		warning("setting ioprio failed: %m (ignored)");
 
 	ret = ioctl(sp->fd, BTRFS_IOC_SCRUB, &sp->scrub_args);
 	gettimeofday(&tv, NULL);
@@ -1175,19 +1172,16 @@ static int scrub_start(int argc, char **argv, int resume)
 		case 'f':
 			force = 1;
 			break;
-		case '?':
 		default:
-			usage(resume ? cmd_scrub_resume_usage :
-						cmd_scrub_start_usage);
+			usage_unknown_option(resume ? cmd_scrub_resume_usage :
+						cmd_scrub_start_usage, argv);
 		}
 	}
 
 	/* try to catch most error cases before forking */
 
-	if (check_argc_exact(argc - optind, 1)) {
-		usage(resume ? cmd_scrub_resume_usage :
-					cmd_scrub_start_usage);
-	}
+	if (check_argc_exact(argc - optind, 1))
+		return 1;
 
 	spc.progress = NULL;
 	if (do_quiet && do_print)
@@ -1195,8 +1189,8 @@ static int scrub_start(int argc, char **argv, int resume)
 
 	if (mkdir_p(datafile)) {
 		warning_on(!do_quiet,
-    "cannot create scrub data file, mkdir %s failed: %s. Status recording disabled",
-			datafile, strerror(errno));
+    "cannot create scrub data file, mkdir %s failed: %m. Status recording disabled",
+			datafile);
 		do_record = 0;
 	}
 	free(datafile);
@@ -1209,9 +1203,9 @@ static int scrub_start(int argc, char **argv, int resume)
 
 	ret = get_fs_info(path, &fi_args, &di_args);
 	if (ret) {
+		errno = -ret;
 		error_on(!do_quiet,
-			"getting dev info for scrub failed: %s",
-			 strerror(-ret));
+			"getting dev info for scrub failed: %m");
 		err = 1;
 		goto out;
 	}
@@ -1224,13 +1218,14 @@ static int scrub_start(int argc, char **argv, int resume)
 	uuid_unparse(fi_args.fsid, fsid);
 	fdres = scrub_open_file_r(SCRUB_DATA_FILE, fsid);
 	if (fdres < 0 && fdres != -ENOENT) {
-		warning_on(!do_quiet, "failed to open status file: %s",
-			strerror(-fdres));
+		errno = -fdres;
+		warning_on(!do_quiet, "failed to open status file: %m");
 	} else if (fdres >= 0) {
 		past_scrubs = scrub_read_file(fdres, !do_quiet);
-		if (IS_ERR(past_scrubs))
-			warning_on(!do_quiet, "failed to read status file: %s",
-				strerror(-PTR_ERR(past_scrubs)));
+		if (IS_ERR(past_scrubs)) {
+			errno = -PTR_ERR(past_scrubs);
+			warning_on(!do_quiet, "failed to read status file: %m");
+		}
 		close(fdres);
 	}
 
@@ -1267,7 +1262,7 @@ static int scrub_start(int argc, char **argv, int resume)
 	spc.progress = calloc(fi_args.num_devices * 2, sizeof(*spc.progress));
 
 	if (!t_devs || !sp || !spc.progress) {
-		error_on(!do_quiet, "scrub failed: %s", strerror(errno));
+		error_on(!do_quiet, "scrub failed: %m");
 		err = 1;
 		goto out;
 	}
@@ -1276,8 +1271,8 @@ static int scrub_start(int argc, char **argv, int resume)
 		devid = di_args[i].devid;
 		ret = pthread_mutex_init(&sp[i].progress_mutex, NULL);
 		if (ret) {
-			error_on(!do_quiet, "pthread_mutex_init failed: %s",
-				strerror(ret));
+			errno = ret;
+			error_on(!do_quiet, "pthread_mutex_init failed: %m");
 			err = 1;
 			goto out;
 		}
@@ -1346,9 +1341,9 @@ static int scrub_start(int argc, char **argv, int resume)
 		ret = listen(prg_fd, 100);
 	if (ret == -1) {
 		warning_on(!do_quiet,
-   "failed to open the progress status socket at %s: %s. Progress cannot be queried",
+   "failed to open the progress status socket at %s: %m. Progress cannot be queried",
 			sock_path[0] ? sock_path :
-			SCRUB_PROGRESS_SOCKET_PATH, strerror(errno));
+			SCRUB_PROGRESS_SOCKET_PATH);
 		if (prg_fd != -1) {
 			close(prg_fd);
 			prg_fd = -1;
@@ -1362,9 +1357,9 @@ static int scrub_start(int argc, char **argv, int resume)
 		ret = scrub_write_progress(&spc_write_mutex, fsid, sp,
 					   fi_args.num_devices);
 		if (ret) {
+			errno = -ret;
 			warning_on(!do_quiet,
-   "failed to write the progress status file: %s. Status recording disabled",
-				strerror(-ret));
+   "failed to write the progress status file: %m. Status recording disabled");
 			do_record = 0;
 		}
 	}
@@ -1372,8 +1367,7 @@ static int scrub_start(int argc, char **argv, int resume)
 	if (do_background) {
 		pid = fork();
 		if (pid == -1) {
-			error_on(!do_quiet, "cannot scrub, fork failed: %s",
-				strerror(errno));
+			error_on(!do_quiet, "cannot scrub, fork failed: %m");
 			err = 1;
 			goto out;
 		}
@@ -1391,8 +1385,8 @@ static int scrub_start(int argc, char **argv, int resume)
 			}
 			ret = wait(&stat);
 			if (ret != pid) {
-				error_on(!do_quiet, "wait failed (ret=%d): %s",
-					ret, strerror(errno));
+				error_on(!do_quiet, "wait failed (ret=%d): %m",
+					ret);
 				err = 1;
 				goto out;
 			}
@@ -1422,9 +1416,12 @@ static int scrub_start(int argc, char **argv, int resume)
 		ret = pthread_create(&t_devs[i], NULL,
 					scrub_one_dev, &sp[i]);
 		if (ret) {
-			if (do_print)
-			error("creating scrub_one_dev[%llu] thread failed: %s",
-				devid, strerror(ret));
+			if (do_print) {
+				errno = ret;
+				error(
+				"creating scrub_one_dev[%llu] thread failed: %m",
+					devid);
+			}
 			err = 1;
 			goto out;
 		}
@@ -1438,9 +1435,10 @@ static int scrub_start(int argc, char **argv, int resume)
 	spc.fi = &fi_args;
 	ret = pthread_create(&t_prog, NULL, scrub_progress_cycle, &spc);
 	if (ret) {
-		if (do_print)
-			error("creating progress thread failed: %s",
-				strerror(ret));
+		if (do_print) {
+			errno = ret;
+			error("creating progress thread failed: %m");
+		}
 		err = 1;
 		goto out;
 	}
@@ -1452,9 +1450,12 @@ static int scrub_start(int argc, char **argv, int resume)
 		devid = di_args[i].devid;
 		ret = pthread_join(t_devs[i], NULL);
 		if (ret) {
-			if (do_print)
-			  error("pthread_join failed for scrub_one_dev[%llu]: %s",
-				devid, strerror(ret));
+			if (do_print) {
+				errno = ret;
+				error(
+				"pthread_join failed for scrub_one_dev[%llu]: %m",
+					devid);
+			}
 			++err;
 			continue;
 		}
@@ -1469,11 +1470,13 @@ static int scrub_start(int argc, char **argv, int resume)
 				++err;
 				break;
 			default:
-				if (do_print)
-		error("scrubbing %s failed for device id %lld: ret=%d, errno=%d (%s)",
-					path, devid,
-					sp[i].ret, sp[i].ioctl_errno,
-					strerror(sp[i].ioctl_errno));
+				if (do_print) {
+					errno = sp[i].ioctl_errno;
+					error(
+		"scrubbing %s failed for device id %lld: ret=%d, errno=%d (%m)",
+						path, devid, sp[i].ret,
+						sp[i].ioctl_errno);
+				}
 				++err;
 				continue;
 			}
@@ -1515,21 +1518,23 @@ static int scrub_start(int argc, char **argv, int resume)
 
 	/* check for errors from the handling of the progress thread */
 	if (do_print && ret) {
-		error("progress thread handling failed: %s",
-			strerror(ret));
+		errno = ret;
+		error("progress thread handling failed: %m");
 	}
 
 	/* check for errors returned from the progress thread itself */
-	if (do_print && terr && terr != PTHREAD_CANCELED)
-		error("recording progress failed: %s",
-			strerror(-PTR_ERR(terr)));
+	if (do_print && terr && terr != PTHREAD_CANCELED) {
+		errno = -PTR_ERR(terr);
+		error("recording progress failed: %m");
+	}
 
 	if (do_record) {
 		ret = scrub_write_progress(&spc_write_mutex, fsid, sp,
 					   fi_args.num_devices);
-		if (ret && do_print)
-			error("failed to record the result: %s",
-				strerror(-ret));
+		if (ret && do_print) {
+			errno = -ret;
+			error("failed to record the result: %m");
+		}
 	}
 
 	scrub_handle_sigint_child(-1);
@@ -1599,7 +1604,7 @@ static int cmd_scrub_cancel(int argc, char **argv)
 	clean_args_no_options(argc, argv, cmd_scrub_cancel_usage);
 
 	if (check_argc_exact(argc - optind, 1))
-		usage(cmd_scrub_cancel_usage);
+		return 1;
 
 	path = argv[optind];
 
@@ -1688,14 +1693,13 @@ static int cmd_scrub_status(int argc, char **argv)
 		case 'R':
 			print_raw = 1;
 			break;
-		case '?':
 		default:
-			usage(cmd_scrub_status_usage);
+			usage_unknown_option(cmd_scrub_status_usage, argv);
 		}
 	}
 
 	if (check_argc_exact(argc - optind, 1))
-		usage(cmd_scrub_status_usage);
+		return 1;
 
 	path = argv[optind];
 
@@ -1705,8 +1709,8 @@ static int cmd_scrub_status(int argc, char **argv)
 
 	ret = get_fs_info(path, &fi_args, &di_args);
 	if (ret) {
-		error("getting dev info for scrub failed: %s",
-			strerror(-ret));
+		errno = -ret;
+		error("getting dev info for scrub failed: %m");
 		err = 1;
 		goto out;
 	}
@@ -1720,8 +1724,7 @@ static int cmd_scrub_status(int argc, char **argv)
 
 	fdres = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (fdres == -1) {
-		error("failed to create socket to receive progress information: %s",
-			strerror(errno));
+		error("failed to create socket to receive progress information: %m");
 		err = 1;
 		goto out;
 	}
@@ -1734,8 +1737,8 @@ static int cmd_scrub_status(int argc, char **argv)
 		close(fdres);
 		fdres = scrub_open_file_r(SCRUB_DATA_FILE, fsid);
 		if (fdres < 0 && fdres != -ENOENT) {
-			warning("failed to open status file: %s",
-				strerror(-fdres));
+			errno = -fdres;
+			warning("failed to open status file: %m");
 			err = 1;
 			goto out;
 		}
@@ -1743,9 +1746,10 @@ static int cmd_scrub_status(int argc, char **argv)
 
 	if (fdres >= 0) {
 		past_scrubs = scrub_read_file(fdres, 1);
-		if (IS_ERR(past_scrubs))
-			warning("failed to read status: %s",
-				strerror(-PTR_ERR(past_scrubs)));
+		if (IS_ERR(past_scrubs)) {
+			errno = -PTR_ERR(past_scrubs);
+			warning("failed to read status: %m");
+		}
 	}
 	in_progress = is_scrub_running_in_kernel(fdmnt, di_args, fi_args.num_devices);
 
